import math


def multi(x):
    """
    Prints out a multiplication table
    :param x: the base for the table
    :return: nothing
    """
    for i in range(11):
        print('{0:2d} * {1:2d} = {2:3d}'.format(i, x, i * x))





def pathelem(path):
    """
    Prints out particular path elements passed as the argument
    :type path: str
    :param path: the string to be parsed
    :return: nothing
    """
    elements = path.split('/')
    elements.pop(0)
    filename = elements.pop(-1)
    if '.' not in filename:
        elements.append(filename)
        filename = ''

    for i in elements:
        print(i)

    print()
    if filename != '':
        print('Filename:  ', filename.split('.')[0])
        print('Extension: ', filename.split('.')[1])


def nnn(n):
    """
    Prints value of n+nn+nnn for a given n. Note here that n is string,
    so 3 * n is nnn (still a string) that is eventually transformed to
    int.
    :type n: str
    :param n: the input number
    :return: the value of n+nn+nnn
    """
    n1 = int(3 * n)
    n2 = int(2 * n)
    n3 = int(n)
    print(n1 + n2 + n3)


def erasto(n):
    """
    Applies the Eratosthenes algorithm up to number n
    :param n: the high limit on the numbers
    :return: the list of primes up to n
    """
    threshold = int(math.sqrt(n))

    ints = list(range(n + 1))

    # removing 0 and 1
    ints.pop(0)
    ints.pop(0)

    i = 2
    idx = 1

    while i <= threshold:
        for j in range(2, n // i + 1):
            try:
                ints.remove(i * j)
            except ValueError:
                pass

        i = ints[idx]
        idx = idx + 1

    print(ints)

    return ints


def countprimes(low, high):
    """
    Prints the primes between low and high using the Eratosthenes algorithm.
    In case the low argument is less than the high argument, the arguments are swapped.

    Keyword arguments:
    low -- the first limit of the interval
    high -- the second limit of the interval

    Returns: Nothing
    """
    if low > high:
        low, high = high, low

    allprimes = erasto(high)

    filtered = [i for i in allprimes if i >= low]

    print(f'Number of primes between {low} and {high}: {len(filtered)}')
    print(filtered)



def perms(s, pref = ''):
    """
    Computes and prints the permutations of string @param s with the given prefix @pref
    :param pref: the constant prefix that will be printed out in front of each string
    :param s: the string whose permutations are generated and printed
    :return: nothing
    """
    if len(s) == 1:
        print(pref + s)
    else:
        for i in range(len(s)):
            c = s[i]
            perms(s[:i] + s[i + 1:], pref + c)


def printtreeline(linelevel, number, level):
    for i in range(number):
        if linelevel == 0:
            print((2 ** (level - 1) - 1) * ' ', end='')
            print('*', end='')
            print((2 ** (level - 1)) * ' ', end='')
        else:
            print((2 ** (level - 1) - linelevel - 1) * ' ', end='')
            print('*', end='')
            print((linelevel * 2 - 1) * ' ', end='')
            print('*', end='')
            print((2 ** (level - 1) - linelevel) * ' ', end='')

    print()


def printsubtrees(level, number):
    if level > 2:
        for j in range(2 ** (level - 2)):
            printtreeline(j, number, level)

        printsubtrees(level - 1, number * 2)

    else:
        printtreeline(0, number, 2)
        printtreeline(0, number * 2, 1)


def printtree(level):
    if level == 0:
        print()
    elif level == 1:
        print('*')
    else:
        printsubtrees(level, 1)


multi(10)
pathelem('/home/user/dir1/file.txt')
nnn('15')
countprimes(9900, 10000)
perms('abcd')
printtree(5)
